/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.List;
import javax.swing.*;

import com.wraithbane.dicom.*;

public class AnonymiseDicom extends JFrame
{
    protected JMenuBar _appMenuBar;
    protected JMenu _fileMenu;
    protected JMenu _helpMenu;
    protected JMenuItem _openFilesItem;
    protected JMenuItem _anonymiseItem;
    protected JMenuItem _exitItem;
    protected JMenuItem _aboutItem;
    protected JFileChooser _fileChooser;
    
    protected Container _contentPane;
    protected GridBagConstraints _gbc;
    protected JLabel _labels[];
    protected JLabel _nameStubLabel;
    protected JTextField _nameStubText;
    protected JTextField _textFields[];
    protected int _nFields;
    protected anonField _fields[];

    protected File _fileList[];
    protected int _nFiles;
    protected String _nameStub = "anon";
    
    private static String _appName = "AnonymiseDicom";
    private static String _appVer = "0.1";
    
    /** Creates a new instance of AnonymiseDicom */
    public AnonymiseDicom()
    {
        int i;
        
        _nFields = 2;
        _fields = new anonField[_nFields];
        for (i=0; i<_nFields; i++)
        {
            _fields[i] = new anonField();
        }
        _fields[0].tag = 0x0010;
        _fields[0].descr = "Name";
        _fields[0].vr = "PN";
        _fields[1].tag = 0x0020;
        _fields[1].descr = "ID";
        _fields[1].vr = "LO";
        
        initComponents();
    }

    protected void initComponents()
    {
        int i;
        GridBagConstraints gbc = new GridBagConstraints();
        
        Font uiFont = new Font("Dialog", Font.PLAIN, 12);
        setFont(uiFont);
        
        // Build the menu bar
        _appMenuBar = new JMenuBar();
        setJMenuBar(_appMenuBar);
        _appMenuBar.setFont(uiFont);

        // File menu
        _fileMenu =  new JMenu("File");
        _fileMenu.setMnemonic('F');
        _fileMenu.setFont(uiFont);
        _openFilesItem = new JMenuItem("Open Files...");
        _openFilesItem.setMnemonic('O');
        _openFilesItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        _openFilesItem.setFont(uiFont);
        _anonymiseItem = new JMenuItem("Anonymise");
        _anonymiseItem.setMnemonic('n');
        _anonymiseItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        _anonymiseItem.setFont(uiFont);
        _exitItem = new JMenuItem("Exit");
        _exitItem.setMnemonic('x');
        _exitItem.setFont(uiFont);
        _fileMenu.add(_openFilesItem);
        _fileMenu.add(_anonymiseItem);
        _fileMenu.addSeparator();
        _fileMenu.add(_exitItem);
        
        // Help menu
        _helpMenu = new JMenu("Help");
        _helpMenu.setMnemonic('H');
        _helpMenu.setFont(uiFont);
        _aboutItem = new JMenuItem("About");
        _helpMenu.add(_aboutItem);
        _aboutItem.setMnemonic('A');
        _aboutItem.setFont(uiFont);
        
        // Add menus to menubar
        _appMenuBar.add(_fileMenu);
        _appMenuBar.add(_helpMenu);
        
        // Add the File Chooser
        _fileChooser = new JFileChooser();
        // Allow multiple selection
        _fileChooser.setMultiSelectionEnabled(true);
        
        // Layout the labels and text fields
        _contentPane = getContentPane();
        _contentPane.setLayout(new GridBagLayout());
        gbc.fill = GridBagConstraints.HORIZONTAL;
        _nameStubLabel = new JLabel(" File Prefix ");
        _nameStubText = new JTextField(_nameStub, 24);
        gbc.gridx = gbc.gridy = 0;
        _contentPane.add(_nameStubLabel, gbc);
        gbc.gridx = 1;
        _contentPane.add(_nameStubText, gbc);
        _labels = new JLabel[_nFields];
        _textFields = new JTextField[_nFields];
        for (i=0; i<_nFields; i++)
        {
            _labels[i] = new JLabel(" "+_fields[i].descr+"  ");
            _textFields[i] = new JTextField(24);
            gbc.gridx = 0;
            gbc.gridy = i+1;
            _contentPane.add(_labels[i], gbc);
            gbc.gridx = 1;
            _contentPane.add(_textFields[i], gbc);
        }
        
        // Finalise layout
        setTitle(_appName);
        pack();
        
        // Add ActionListeners to components
        _openFilesItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onOpenFilesItem(evt);
            }
        });

        _anonymiseItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onAnonymiseItem(evt);
            }
        });

        _exitItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onExitItem(evt);
            }
        });

        _aboutItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onAboutItem(evt);
            }
        });

        // WindowAdapter so the app behaves itself
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                System.exit(0);
            }
        });
    }
    
    public static void main(String args[])
    {
        new AnonymiseDicom().show();
    }

    protected void onOpenFilesItem(java.awt.event.ActionEvent evt)
    {
        int iChosen, i;
        DicomFile dcmFile = new DicomFile();
        String dcmFileName;
        
        iChosen = _fileChooser.showOpenDialog(this);
        
        if (iChosen == JFileChooser.APPROVE_OPTION)
        {
            File fileListTemp[] = _fileChooser.getSelectedFiles();
            _nFiles = fileListTemp.length;
            _fileList = new File[_nFiles];
            for (i=0; i<_nFiles; i++)
            {
                _fileList[i] = fileListTemp[i];
            }

            try
            {
                dcmFileName = _fileList[0].getAbsolutePath();
                dcmFile = new DicomFile(dcmFileName);
            }
            catch (Exception exDCM)
            {
                System.out.println(exDCM.toString());
                System.exit(1);
            }
            for (i=0; i<_nFields; i++)
            {
                try
                {
                    _textFields[i].setText(dcmFile.getValue(_fields[i].group, 
                        _fields[i].tag, Dicom.PN));
                }
                catch (Exception exDCM)
                {
                    System.out.println(exDCM.toString());
                    System.exit(1);
                }
            }
        }
    }

    protected void onAnonymiseItem(java.awt.event.ActionEvent evt)
    {
        String currFile, currDir, nameStub, newFileName, prefix, suffix;
        DicomFile dcmFile;
        DicomGroup dcmGroup;
        DicomElement dcmElement;
        FileInputStream inStream;
        FileOutputStream outStream;
        File outFile;
        int i, j, k;
        int bytesToCopy, dataLen, elemOffset, valueLen;
        int writeOffset, xferSyntax;
        byte bData[], bBuffer[];
        boolean overwriteAll = false;
        
        // Where are the files?
        currDir = _fileList[0].getParent().concat(File.separator);
        
        // generate prefix and suffix of new filename
        nameStub = _nameStubText.getText();
        if (nameStub.length() == 0)
        {
            nameStub = "anon";
        }
        prefix = currDir.concat(nameStub).concat("_");
        suffix = new String(".dcm");
        
        // Loop through each file
        for (i=0; i<_nFiles; i++)
        {
            // Haul the file data into RAM
            dataLen = (int)_fileList[i].length();
            bData = new byte[dataLen];
            try
            {
                inStream = new FileInputStream(_fileList[i]);
                inStream.read(bData, 0, dataLen);
                inStream.close();
            }
            catch (IOException exIO)
            {
                System.out.println(exIO.toString());
                continue;
            }
            
            // Load the file and change all the fields in bData
            dcmFile = new DicomFile();
            try
            {
                // Read it again as a DICOM file
                dcmFile.read(_fileList[i].getAbsolutePath());
                xferSyntax = dcmFile.getXFerSyntax();

                // Find each element to change
                for (j=0; j<_nFields; j++)
                {
                    dcmGroup = dcmFile.getGroup(_fields[j].group);
                    elemOffset = dcmGroup.getOffsetToElement(_fields[j].tag);
                    dcmElement = new DicomElement(bData, elemOffset,
                        xferSyntax);
                    valueLen = dcmElement.getValueLen();
                    writeOffset = elemOffset + dcmElement.getLength() - 
                        valueLen;
                    // Check how to write data
                    switch (xferSyntax)
                    {
                        case Dicom.XFER_IMP_VR_LITTLE_ENDIAN:
                        case Dicom.XFER_EXP_VR_LITTLE_ENDIAN:
                        {
                            bBuffer = _textFields[j].getText().getBytes();
                            bytesToCopy = _textFields[j].getText().length();
                            // New value is too long => chop it off silently
                            if (bytesToCopy > valueLen)
                            {
                                bytesToCopy = valueLen;
                            }
                            // Copy new value into bData
                            for (k=0; k<bytesToCopy; k++)
                            {
                                bData[writeOffset+k] = bBuffer[k];
                            }
                            // pad with spaces
                            for (k=bytesToCopy; k<valueLen; k++)
                            {
                                bData[writeOffset+k] = 20;
                            }
                            break;
                        }
                        
                        case Dicom.XFER_EXP_VR_BIG_ENDIAN:
                        {
                            break;
                        }
                        
                        default:
                    }
                }
            }
            catch (Exception ex)
            {
                System.out.println(ex.toString());
                continue;
            }
            
            // new file
            newFileName = prefix.concat(Integer.toString(i)).concat(suffix);
            outFile = new File(newFileName);
            try
            {
                if (!outFile.createNewFile())
                {
                    // Just delete it
                    if (!(outFile.delete() && outFile.createNewFile()))
                    {
                        // FFS, give up and throw exception
                        throw new IOException("File "+newFileName+" can't be overwritten.");
                    }
                }
                outStream = new FileOutputStream(outFile);
                outStream.write(bData, 0, dataLen);
                outStream.close();
            }
            catch (IOException exIO)
            {
                System.out.println(exIO.toString());
                continue;
            }
        }
        JOptionPane.showMessageDialog(this, "Anonymisation complete", _appName,
            JOptionPane.INFORMATION_MESSAGE);
    }

    protected void onExitItem(java.awt.event.ActionEvent evt)
    {
        System.exit(0);
    }

    protected void onAboutItem(java.awt.event.ActionEvent evt)
    {
        JOptionPane.showMessageDialog(this, 
            _appName.concat(" v").concat(_appVer).concat("\n\n(c) James d'Arcy 2003"), 
            _appName, JOptionPane.INFORMATION_MESSAGE);
    }
    
}
