/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.*;

import com.wraithbane.dicom.*;

public class DicomBrowse extends JFrame
{
    protected JMenuBar _appMenuBar;
    protected JMenu _fileMenu;
    protected JMenu _helpMenu;
    protected JMenuItem _openDirItem;
    protected JMenuItem _exitItem;
    protected JMenuItem _aboutItem;
    protected JFileChooser _fileChooser;
    
    protected DisplayCell _dispCell;
    
    private String _appName = "Fenris";
    
    /** Creates a new instance of DicomBrowse */
    public DicomBrowse()
    {
        initComponents();
    }

    protected void initComponents()
    {
        Font uiFont = new Font("Dialog", Font.PLAIN, 12);
        setFont(uiFont);
        
        // Build the menu bar
        _appMenuBar = new JMenuBar();
        setJMenuBar(_appMenuBar);
        _appMenuBar.setFont(uiFont);

        // File menu
        _fileMenu =  new JMenu("File");
        _fileMenu.setMnemonic('F');
        _fileMenu.setFont(uiFont);
        _openDirItem = new JMenuItem("Open Directory...");
        _openDirItem.setMnemonic('O');
        _openDirItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        _openDirItem.setFont(uiFont);
        _exitItem = new JMenuItem("Exit");
        _exitItem.setMnemonic('x');
        _exitItem.setFont(uiFont);
        _fileMenu.add(_openDirItem);
        _fileMenu.addSeparator();
        _fileMenu.add(_exitItem);
        
        // Help menu
        _helpMenu = new JMenu("Help");
        _helpMenu.setMnemonic('H');
        _helpMenu.setFont(uiFont);
        _aboutItem = new JMenuItem("About");
        _helpMenu.add(_aboutItem);
        _aboutItem.setMnemonic('A');
        _aboutItem.setFont(uiFont);
        
        // Add menus to menubar
        _appMenuBar.add(_fileMenu);
        _appMenuBar.add(_helpMenu);
        
        // Add the File Chooser
        _fileChooser = new JFileChooser();
        
        // Add the image display elements
        _dispCell = new DisplayCell();
        getContentPane().add(_dispCell);
        _dispCell.setActive();
        
        // Finalise layout
        setTitle(_appName);
        pack();
        
        // Add ActionListeners to components
        _openDirItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onOpenDirItem(evt);
            }
        });

        _exitItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onExitItem(evt);
            }
        });

        _aboutItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                onAboutItem(evt);
            }
        });

        // WindowAdapter so the app behaves itself
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                System.exit(0);
            }
        });
    }
    
    public static void main(String args[])
    {
        new DicomBrowse().show();
    }

    protected void onOpenDirItem(java.awt.event.ActionEvent evt)
    {
        File file;
        int iChosen;
        DicomFile dcmFile;
        
        iChosen = _fileChooser.showOpenDialog(this);
        
        if (iChosen == JFileChooser.APPROVE_OPTION)
        {
            file = _fileChooser.getSelectedFile();
            dcmFile = new DicomFile();
            try
            {
                dcmFile.read(file.getAbsolutePath());
            }
            catch (IOException exIO)
            {
                System.out.println(exIO.toString());
                return;
            }
        
            try
            {
/*                System.out.println(dcmFile.getValue(0x0010, 0x0010, Dicom.PN));
                System.out.println("Series: "+dcmFile.getValue(0x0020, 0x0011, Dicom.IS)+"\nImage: "+
                    dcmFile.getValue(0x0020, 0x0013, Dicom.IS));
*/
                _dispCell.setPixelData(dcmFile.getArrValue(0x7fe0, 0x0010, Dicom.OW),
                    dcmFile.getValue(0x0028, 0x0010, Dicom.US),
                    dcmFile.getValue(0x0028, 0x0011, Dicom.US));
                int centre = Math.round(Float.parseFloat(dcmFile.getValue(0x0028, 0x1050, Dicom.DS)));
                int width = Math.round(Float.parseFloat(dcmFile.getValue(0x0028, 0x1051, Dicom.DS)));
                _dispCell.setWindowLevel(centre, width);
                setAnnotation(dcmFile, _dispCell);
            }
            catch (Exception exDCM)
            {
                System.out.println(exDCM.toString());
            }
        }
    }

    protected void onExitItem(java.awt.event.ActionEvent evt)
    {
        System.exit(0);
    }

    protected void onAboutItem(java.awt.event.ActionEvent evt)
    {
        System.out.println("Help.About selected");
    }
    
    protected void setAnnotation(DicomFile dcmFile, DisplayCell dispCell)
    {
        String[] strValue = new String[5];
        StringTokenizer tokenizer;
        short nX, nY;
        float pixSpaceX, pixSpaceY, posX, posY, posZ;
        int dotIndex, fov;
        
        try
        {
            // Patient data
            strValue[0] = dcmFile.getValue(0x0010, 0x0010, Dicom.PN);
            strValue[1] = dcmFile.getValue(0x0010, 0x0020, Dicom.LO);
            strValue[2] = dcmFile.getValue(0x0010, 0x0030, Dicom.DA);
            strValue[2] = Dicom.prettifyDate(strValue[2]);
            dispCell.setPatientData(strValue[0], strValue[1], strValue[2]);
            
            // Acquisition stuff
            strValue[0] = dcmFile.getValue(0x0018, 0x0080, Dicom.DS);
            dotIndex = strValue[0].indexOf(".");
            if (dotIndex != -1)
            {
                strValue[0] = strValue[0].substring(0, dotIndex+2);
            }
            strValue[1] = dcmFile.getValue(0x0018, 0x0081, Dicom.DS);
            dotIndex = strValue[1].indexOf(".");
            if (dotIndex != -1)
            {
                strValue[1] = strValue[1].substring(0, dotIndex+2);
            }
            nX = dcmFile.getValue(0x0028, 0x0010, Dicom.US);
            nY = dcmFile.getValue(0x0028, 0x0011, Dicom.US);
            strValue[2] = Short.toString(nX);
            strValue[2] = strValue[2].concat("x");
            strValue[2] = strValue[2].concat(Short.toString(nY));
            strValue[3] = dcmFile.getValue(0x0028, 0x0030, Dicom.DS);
            tokenizer = new StringTokenizer(strValue[3], "\\");
            pixSpaceX = Float.parseFloat(tokenizer.nextToken());
            pixSpaceY = Float.parseFloat(tokenizer.nextToken());
            strValue[3] = Integer.toString(Math.round(pixSpaceX*nX));
            dispCell.setAcquisitionData(strValue[0], strValue[1], strValue[2], 
                strValue[3]);
            
            // Series data
            strValue[0] = dcmFile.getValue(0x0008, 0x103e, Dicom.DS);
            strValue[1] = Dicom.prettifyDate(dcmFile.getValue(0x0008, 0x0020, Dicom.DA));
            strValue[2] = dcmFile.getValue(0x0008, 0x0031, Dicom.TM);
            dotIndex = strValue[2].indexOf(".");
            if (dotIndex != -1)
            {
                strValue[2] = strValue[2].substring(0, dotIndex);
            }
            strValue[3] = dcmFile.getValue(0x0020, 0x0011, Dicom.IS);
            strValue[4] = dcmFile.getValue(0x0020, 0x0013, Dicom.IS);
            dispCell.setSeriesData(strValue[0], strValue[1], strValue[2], 
                strValue[3], strValue[4]);
            
            // Spatial data
            strValue[0] = dcmFile.getValue(0x0018, 0x0050, Dicom.DS);
            dotIndex = strValue[0].indexOf(".");
            if (dotIndex != -1)
            {
                strValue[0] = strValue[0].substring(0, dotIndex+2);
            }
            strValue[1] = dcmFile.getValue(0x0020, 0x0032, Dicom.DS);
            tokenizer = new StringTokenizer(strValue[1], "\\");
            strValue[2] = dcmFile.getValue(0x0020, 0x0037, Dicom.DS);

            dispCell.setSpatialData(strValue[0], strValue[1], strValue[2]);
        }
        catch(Exception exDCM)
        {
            System.out.println(exDCM.toString());
        }
    }

}
