/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ImageAnnotation extends JPanel
{
    
    protected String _patientName;
    protected String _patientID;
    protected String _dateOfBirth;
    protected String _studyDate;
    protected String _slicePos;
    protected String _sliceThick;
    protected String _sliceOrient;
    protected String _tr;
    protected String _te;
    protected String _seriesDescr;
    protected String _seriesTime;
    protected String _seriesNum;
    protected String _imaNum;
    protected String _matrix;
    protected String _fieldOfView;

    protected String _winCentre;
    protected String _winWidth;
    
    protected Color _fontColour;
    protected FontMetrics _fontMetrics;
    protected int _fontHeight;
    protected int _fontAscent;
    protected int _fontMaxAdvance;
    protected int _textInset = 2;
    
    protected boolean _active;
    
    /** Creates a new instance of ImageAnnotation */
    public ImageAnnotation()
    {
        // Render the text as an overlay
        setOpaque(false);
        
        // Set up font
        _fontColour = Color.green;
        Font font = new Font("Dialog", Font.PLAIN, 12);
        setFont(font);
        
    }
    
    public void paintComponent(Graphics g)
    {
        int drawPosX, drawPosY;
        super.paintComponent(g);
        
        if (_active)
        {
            g.setColor(_fontColour);
            
            // Top left
            drawPosY = _textInset+_fontAscent;
            g.drawString(_patientName, _textInset, drawPosY);
            drawPosY += _fontHeight;
            g.drawString(_patientID, _textInset, drawPosY);
            drawPosY += _fontHeight;
            g.drawString(_dateOfBirth, _textInset, drawPosY);
            
            // Bottom left
            drawPosY = getHeight() - (_textInset + _fontHeight - _fontAscent);
            g.drawString("TE ", _textInset, drawPosY);
            drawPosX = _textInset + 7*_fontMaxAdvance - 
                _fontMetrics.stringWidth(_te);
            g.drawString(_te, drawPosX, drawPosY);
            drawPosY -= _fontHeight;
            g.drawString("TR ", _textInset, drawPosY);
            drawPosX = _textInset + 7*_fontMaxAdvance - 
                _fontMetrics.stringWidth(_tr);
            g.drawString(_tr, drawPosX, drawPosY);
            drawPosY -= 2*_fontHeight;
            drawPosX = _textInset;
            g.drawString(_sliceOrient, drawPosX, drawPosY);
            drawPosY -= _fontHeight;
            g.drawString("Pos ", _textInset, drawPosY);
            drawPosX = _textInset + 7*_fontMaxAdvance - 
                _fontMetrics.stringWidth(_slicePos);
            g.drawString(_slicePos, drawPosX, drawPosY);
            drawPosY -= _fontHeight;
            g.drawString("Thick ", _textInset, drawPosY);
            drawPosX = _textInset + 7*_fontMaxAdvance - 
                _fontMetrics.stringWidth(_sliceThick);
            g.drawString(_sliceThick, drawPosX, drawPosY);
            
            // Bottom right
            drawPosY = getHeight() - (_textInset + _fontHeight - _fontAscent);
            drawPosX = getWidth() - (_textInset + 6*_fontMaxAdvance);
            g.drawString("C ", drawPosX, drawPosY);
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_winCentre));
            g.drawString(_winCentre, drawPosX, drawPosY);
            drawPosY -= _fontHeight;
            drawPosX = getWidth() - (_textInset + 6*_fontMaxAdvance);
            g.drawString("W ", drawPosX, drawPosY);
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_winWidth));
            g.drawString(_winWidth, drawPosX, drawPosY);
            drawPosY -= 2*_fontHeight;
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_matrix));
            g.drawString(_matrix, drawPosX, drawPosY);
            drawPosY -= _fontHeight;
            drawPosX = getWidth() - (_textInset + 6*_fontMaxAdvance);
            g.drawString("FoV ", drawPosX, drawPosY);
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_fieldOfView));
            g.drawString(_fieldOfView, drawPosX, drawPosY);

            // top right
            drawPosY = _textInset + _fontAscent;
            drawPosX = getWidth() -
                (_textInset + _fontMetrics.stringWidth(_seriesDescr));
            g.drawString(_seriesDescr, drawPosX, drawPosY);
            drawPosY += _fontHeight;
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_studyDate));
            g.drawString(_studyDate, drawPosX, drawPosY);
            drawPosY += _fontHeight;
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_seriesTime));
            g.drawString(_seriesTime, drawPosX, drawPosY);
            drawPosY += _fontHeight;
            drawPosX = getWidth() - (_textInset + 6*_fontMaxAdvance);
            g.drawString("Ser ", drawPosX, drawPosY);
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_seriesNum));
            g.drawString(_seriesNum, drawPosX, drawPosY);
            drawPosY += _fontHeight;
            drawPosX = getWidth() - (_textInset + 6*_fontMaxAdvance);
            g.drawString("Ima ", drawPosX, drawPosY);
            drawPosX = getWidth() - 
                (_textInset + _fontMetrics.stringWidth(_imaNum));
            g.drawString(_imaNum, drawPosX, drawPosY);
        }
    }
    
    public void setActive()
    {
        _active = true;
    }
    
    public void setAcquisitionData(String tr, String te, String matrix, 
        String fieldOfView)
    {
        _tr = tr;
        _te = te;
        _matrix = matrix;
        _fieldOfView = fieldOfView;
    }
    
    public void setFontColour(Color fontColour)
    {
        _fontColour = fontColour;
    }
    
    public void setFont(Font font)
    {
        super.setFont(font);
        
        // Recalculate sizes
        _fontMetrics = getFontMetrics(font);
        _fontAscent = _fontMetrics.getAscent();
        _fontHeight = _fontMetrics.getHeight();
        _fontMaxAdvance = _fontMetrics.stringWidth("O");
    }

    public void setInactive()
    {
        _active = false;
    }
    
    public void setPatientData(String patientName, String patientID,
        String dateOfBirth)
    {
        _patientName = patientName;
        _patientID = patientID;
        _dateOfBirth = dateOfBirth;
    }
    
    public void setSeriesData(String seriesDescr, String studyDate, 
        String seriesTime, String seriesNum, String imaNum)
    {
        _seriesDescr = seriesDescr;
        _studyDate = studyDate;
        _seriesTime = seriesTime;
        _seriesNum = seriesNum;
        _imaNum = imaNum;
    }
        
    public void setSpatialData(String sliceThick, String slicePos, 
        String sliceOrient)
    {
        _sliceThick = sliceThick;
        _slicePos = slicePos;
        _sliceOrient = sliceOrient;
    }
        
    public void setWindowLevel(int centre, int width)
    {
        _winCentre = Integer.toString(centre);
        _winWidth = Integer.toString(width);
    }

}
